/*
 * Copyright (C) 2012 OUYA, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package tv.ouya.sample;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;
import tv.ouya.console.api.CancelIgnoringOuyaResponseListener;
import tv.ouya.console.api.OuyaController;
import tv.ouya.console.api.OuyaEncryptionHelper;
import tv.ouya.console.api.OuyaFacade;
import tv.ouya.console.api.OuyaResponseListener;
import tv.ouya.console.api.Product;
import tv.ouya.console.api.Purchasable;
import tv.ouya.console.api.Receipt;
import tv.ouya.console.internal.util.Strings;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import static tv.ouya.console.api.OuyaController.BUTTON_O;

public class IapSampleActivity extends Activity {
    /**
     * Log onto the developer website (you should have received a URL, a username and a password in email)
     * and get your developer ID. Plug it in here. Use your developer ID, not your developer UUID.
     * <p/>
     * The current value is just a sample developer account. You should change it.
     */
    public static final String DEVELOPER_ID = "310a8f51-4d6e-4ae5-bda0-b93878e5f5d0";

    /**
     * Before this app will run, you must define some purchasable items on the developer website. Once
     * you have defined those items, put their Product IDs in the List below.
     * <p/>
     * The Product IDs below are those in our developer account. You should change them.
     */
    public static final List<Purchasable> PRODUCT_IDENTIFIER_LIST = Arrays.asList(new Purchasable("long_sword"), new Purchasable("sharp_axe"), new Purchasable("__DECLINED__THIS_PURCHASE"));

    /**
     * The receipt adapter will display a previously-purchased item in a cell in a ListView. It's not part of the in-app
     * purchase API. Neither is the ListView itself.
     */
    private ReceiptAdapter receiptAdapter;
    private ListView receiptListView;

    /**
     * Your game talks to the OuyaFacade, which hides all the mechanics of doing an in-app purchase.
     */
    private OuyaFacade ouyaFacade = OuyaFacade.getInstance();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        ouyaFacade.init(this, DEVELOPER_ID);
        setContentView(R.layout.sample_app);

        receiptAdapter = new ReceiptAdapter(this);
        receiptListView = (ListView) findViewById(R.id.receipts);
        receiptListView.setFocusable(false);
        receiptListView.setAdapter(receiptAdapter);

        /*
         * In order to avoid "application not responding" popups, Android demands that long-running operations
         * happen on a background thread. Listener objects provide a way for you to specify what ought to happen
         * at the end of the long-running operation. Examples of this pattern in Android include
         * android.os.AsyncTask.
         */
        OuyaResponseListener<ArrayList<Product>> productListListener = new CancelIgnoringOuyaResponseListener<ArrayList<Product>>() {
            @Override
            public void onSuccess(ArrayList<Product> products) {
                addProducts(products);
            }

            @Override
            public void onFailure(int errorCode, String errorMessage, Bundle optionalData) {
                // Your app probably wants to do something more sophisticated than popping a Toast. This is
                // here to tell you that your app needs to handle this case: if your app doesn't display
                // something, the user won't know of the failure.
                Toast.makeText(IapSampleActivity.this, "Could not fetch product information (error " + errorCode + ": " + errorMessage + ")", Toast.LENGTH_LONG).show();
            }
        };
        findViewById(R.id.gamer_uuid_button).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                ouyaFacade.requestGamerUuid(new CancelIgnoringOuyaResponseListener<String>() {
                    @Override
                    public void onSuccess(String result) {
                        new AlertDialog.Builder(IapSampleActivity.this)
                                .setTitle(getString(R.string.alert_title))
                                .setMessage(result)
                                .show();
                    }

                    @Override
                    public void onFailure(int errorCode, String errorMessage, Bundle optionalData) {
                        Toast.makeText(IapSampleActivity.this, "Unable to fetch gamer UUID (error " + errorCode + ": " + errorMessage + ")", Toast.LENGTH_LONG).show();
                    }
                });
            }
        });
        ouyaFacade.requestProductList(PRODUCT_IDENTIFIER_LIST, productListListener);
        fetchReceipts();
    }

    private void fetchReceipts() {
        ouyaFacade.requestReceipts(new CancelIgnoringOuyaResponseListener<String>() {
            @Override
            public void onSuccess(String receiptResponse) {
                OuyaEncryptionHelper helper = new OuyaEncryptionHelper();
                List<Receipt> receipts = null;
                try {
                    receipts = helper.decryptReceiptResponse(receiptResponse);
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
                Collections.sort(receipts, new Comparator<Receipt>() {
                    @Override
                    public int compare(Receipt lhs, Receipt rhs) {
                        return rhs.getPurchaseDate().compareTo(lhs.getPurchaseDate());
                    }
                });
                receiptAdapter.set(receipts);
            }

            @Override
            public void onFailure(int errorCode, String errorMessage, Bundle optionalData) {
                // Again, your game should do something more sophisticated.
                Toast.makeText(IapSampleActivity.this, "Could not fetch receipts (error " + errorCode + ": " + errorMessage + ")", Toast.LENGTH_LONG).show();
            }
        });
    }

    /*
     * The IAP Facade registers a broadcast receiver with Android. You should take care to call shutdown(),
     * which unregisters the broadcast receiver, when you're done with the IAP Facade.
     */
    @Override
    protected void onDestroy() {
        ouyaFacade.shutdown();
        super.onDestroy();
    }

    public void addProducts(List<Product> products) {
        for (Product product : products) {
            ((ViewGroup) findViewById(R.id.products)).addView(makeButton(product));
        }
    }

    private View makeButton(Product item) {
        LayoutInflater inflater = LayoutInflater.from(this);
        View view = inflater.inflate(R.layout.product_item, null, false);
        String buttonText = item.getName() + " - " + tv.ouya.sample.util.Strings.formatDollarAmount(item.getPriceInCents());
        Button button = (Button) view.findViewById(R.id.purchase_product_button);
        button.setOnClickListener(new RequestPurchaseClickListener());
        button.setText(buttonText);
        button.setTag(item);
        return view;
    }

    /*
     * This will be called when the user clicks on an item in the ListView.
     */
    public void requestPurchase(final String productId) {
        ouyaFacade.requestPurchase(new Purchasable(productId), new CancelIgnoringOuyaResponseListener<Product>() {
            @Override
            public void onSuccess(Product product) {
                new AlertDialog.Builder(IapSampleActivity.this)
                        .setTitle(getString(R.string.alert_title))
                        .setMessage("You have successfully purchased a " + product.getName() + " for " + Strings.formatDollarAmount(product.getPriceInCents()))
                        .show();
                fetchReceipts();
            }

            @Override
            public void onFailure(int errorCode, String errorMessage, Bundle optionalData) {
                new AlertDialog.Builder(IapSampleActivity.this)
                        .setTitle(getString(R.string.alert_title))
                        .setMessage("Unfortunately, your purchase failed [error code " + errorCode + " (" + errorMessage + ")]. Would you like to try again?")
                        .setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialogInterface, int i) {
                                dialogInterface.dismiss();
                                requestPurchase(productId);
                            }
                        })
                        .setNegativeButton(R.string.cancel, null)
                        .show();
            }
        });
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        // We only interpret key codes at this time to support emulators
        // When using a real controller the key code will not need to be interpreted
        int interpreted = OuyaController.interpret(keyCode);
        if(interpreted == BUTTON_O) {
            View focusedButton = getCurrentFocus();
            focusedButton.performClick();
            return true;
        }
        return super.onKeyUp(keyCode, event);
    }

    public class RequestPurchaseClickListener implements View.OnClickListener {
        @Override
        public void onClick(View v) {
            requestPurchase(((Product) v.getTag()).getIdentifier());
        }
    }
}
